clc; clear; close all
%% Equazioni di Eulero
tfin = 5;       % Tempo di simulazione
prec = 1e-4;    % Precisione integrazione

deg2rad = 180/pi;

% Scegli il moto
user = menu('Scegli un caso:','Piccolo disturbo','Moto retrogrado','Not a Disk','Spirale');
switch user
    case 1
        A = 1; B = 1; C = 30; % I_x, Y_y, I_z
        L = @(t) interp1([0 0.2 0.3 0.4 tfin], ...
                         [0  0  5e3  0   0  ], t, 'linear');
        M = @(t) 0;
        N = @(t) 0;
        INIT = [0; 0; 50; 0; 0; 0]; % omega_x, omega_y, omega_z, psi, theta, phi
    case 2
        A = 1; B = 1; C = 10;
        L = @(t) interp1([0 0.2 0.3 0.4 tfin], ...
                         [0  0  7e2  0   0  ], t, 'linear');
        M = @(t) 0;
        N = @(t) 0;
        INIT = [0; 0; 5; 0; 0; 0];
    case 3
        A = 1; B = 5; C = 10;
        L = @(t) interp1([0 0.2 0.3 0.4 tfin], ...
                         [0  0  3e2  0   0  ], t, 'linear');
        M = @(t) 0;
        N = @(t) 0;
        INIT = [0; 0; 10; 0; 0; 0];
    case 4
        A = 1; B = 1; C = 10;
        L = @(t) 500*t;
        M = @(t) 0;
        N = @(t) 0;
        INIT = [0; 0; 50; 0; 0; 0];
end

% Integrazione
opt = odeset('RelTol', prec, 'AbsTol', prec*ones(1,6));
hW = waitbar(0, 'Integrazione in corso...');
Par = {L M N A B C tfin hW};
[T, Y] = ode45(@eulero, [0 tfin], INIT, opt, Par);
delete(hW)

psi = Y(:,4);
theta = Y(:,5);
phi = Y(:,6);

%% Grafica
% Diagramma angoli di Eulero
scrsz = get(0,'ScreenSize');
figure('position', [0 0 scrsz(3)/2 scrsz(4)])

subplot(3,1,[1 2])
plot(T, psi*deg2rad, ...
     T, mod(theta*deg2rad, 360) - 180, ...
     T, mod(phi*deg2rad, 360))
legend('\psi','\theta','\phi')
title('Angoli di Eulero')

subplot(3,1,3)
plot(T, L(T),'r-', T, M(T),'b--',T, N(T),'k-.')
legend('M_x','M_y','M_z')
title('Legge forzante')

% Animazione del moto
fH = figure('position', [scrsz(3)/2 0 scrsz(3)/2 scrsz(4)]);
% Mov = getframe;
Track = zeros(length(T), 3);

% Linearizzo il tempo
NtL = 300*tfin;
tL = linspace(0, tfin, NtL);    
psi_L = interp1(T, psi, tL);
theta_L = interp1(T, theta, tL);
phi_L = interp1(T, phi, tL);

% Animazione
for i = 1:NtL
    figure(fH)
    clf, hold on, view(3), grid, camproj('perspective')
    title(sprintf('Tempo %.3g di %g', tL(i), tfin))
    % Matrice dei coseni direttori xyz
    dcm = ang2dcm(psi_L(i), theta_L(i), phi_L(i));
    % Asse
    Vdown = dcm * [0; 0; -4]; Vup = dcm * [0; 0; 4];
    plot3([Vup(1) Vdown(1)], [Vup(2) Vdown(2)], [Vup(3) Vdown(3)], ...
          'r', 'linewidth', 2)
    % Cilindro
    [Xcy, Ycy, Zcy] = cylinder(2, 30);
    CylD = dcm * [Xcy(1,:); Ycy(1,:); Zcy(1,:)];
    CylU = dcm * [Xcy(2,:); Ycy(2,:); Zcy(2,:)];
    surf([CylD(1,:); CylU(1,:)], ...
         [CylD(2,:); CylU(2,:)], ...
         [CylD(3,:); CylU(3,:)]);
    % Percorso dell'asse
    Track(i, :) = Vup(:);
    if i <= 150, st = 1; else st = i-150; end
    plot3(Track(st:i,1), Track(st:i,2), Track(st:i,3), 'b')
    axis([-4 4 -4 4 -4 4])
    xlabel('x'), ylabel('y'), zlabel('z')
    drawnow;
    % Mov(i) = getframe(fH);
end

plot3(Track(:,1), Track(:,2), Track(:,3), 'b')

% movie2avi(Mov,'vudei.avi','fps',30)
% movie(Mov,30)